unit bio_custom_msgbox;

    (*
    
        (c) 2009 - Christopher Ramrez
                   blindedbythedark [at] gmail [dot] com
                   Esta unidad es de uso libre. Si puedes mejorarla, 
                   por favor comparte los cambios con la comunidad.
                   
                   Parte del Cdigo fue tomado desde:
                   1. http://www.codeguru.com/cpp/w-p/win32/messagebox/article.php/c10873                  
                      Mi trabaja fue traducir el cdigo C original a Delphi y
                      adaptarlo un poco a la forma de trabajar con Delphi
                      
        EJEMPLO DE USO:
        1. Llamar a la funcin CBTMessagebox de forma similar a como llamaras
           a Messagebox --
           
           La funcin CBTMessagebox introduce un ltimo parmetro que es de tipo
            TCBTCustomBtnLabels, esta TIPO indica las etiquetas personalizada
            que recibiaran todos o algunos de los botones en el Dialogo.
           
            EJEMPLO:
            El usuario ha ejecutado el comando salir sin antes guardar cambios
            en un registro que se encuentra abierto.
              
              var
              MyCustomLabels: TCBTCustomBtnLabels;
              begin
                
                ..
                
                MyCustomLabels.YES := 'Guardar';
                MyCustomLabels.NO := 'No guardar';
                // la propiedad CANCEL no es necesaria establecerla porque
                // la etiqueta predeterminada de este botn es adecuada a este caso.
                  
                case CBTMessagebox(Application.Mainform.Handle,
                            'Guardar cambios antes de salir?',
                            'Guardar?',
                            (MB_YESNOCANCEL or MB_ICONINFORMATION or MB_DEFBUTTON1),
                            MyCustomLabels) of
                     IDYES: Guardar;
                     IDNO: {no hacer nada};
                     IDCANCEL: {evitar que Form cierre};
                end;
              end;
                     
                   
    *)
    
interface 

uses
    Windows, SysUtils;
    
type

    // TCBTCustomBtnLabels: Este tipo de registro se utiliza para indicar que
    //                      etiqueta perzonalizada recibiran los distintos
    //                      botones estndar en un MessageBox. Por ejemplo:
    //                      El miembro ABORT contiene la etiqueta para el botn
    //                      IDABORT, el YES contiene la destina a IDYES y as
    //                      sucesivamente.
    //                      
    //                      NO ES NECESARIO ESTABLECER TODAS LAS PROPIEDADES DE
    //                      LA CLASE. SOLO ESTABLEZCA LA PROPIEDAD CORRESPONDIENTE
    //                      AL BOTN AL QUE DESEA ESTABLECER LA ETIQUETA.
    //
    //        >>> TENGA EN CUENTA LA LONGITUD DEL TEXTO QUE ASIGNARA <<<
    TCBTCustomBtnLabels = record
        ABORT       : String;
        RETRY       : String;
        IGNORE      : String;
        CANCEL      : String;
        TRYAGAIN    : String;
        CONTINUE    : String;
        HELP        : String;
        OK          : String;
        YES         : String;
        NO          : String;
    end;

    // Declaramos las funciones disponibles.
    // Parte del Cdigo fue tomado desde:
    // http://www.codeguru.com/cpp/w-p/win32/messagebox/article.php/c10873
    
    // CBTMessagebox: Es un capa para la funcin MessageBox de la API de Windows.
    //                Esta funcin usa un Hook para cambiar el texto estndar
    //                de los botones que ofrece la API:
    //                (YES, NO, CANCEL... etc.)
    function CBTMessagebox(hWnd: HWND;
                           lpText: PChar;
                           lpCaption: PChar;
                           uType: Cardinal;
                           CustomBtnCaptions: TCBTCustomBtnLabels): Cardinal;
                  
    
implementation

var
    hhk: HHOOK;
    fCustomBtnsLabels: TCBTCustomBtnLabels;

// CBTProc: Este procedimiento se utiliza para recibir las notificaciones
//          del sistema acerca de la creacin de un nuevo "msgdialog",
//          "MessageBox". En este procedimiento es donde le damos
//          intrucciones al sistema de reemplazar el texto de los botones.
//
//          Este  procedimiento hace uso de la varible privada
//          fCustomBtnsLabels, la que debe estar asignada antes de 
//          que est funcin sea asignada como un Hook.

function CBTProc(nCode: Integer;
                 wParam: WPARAM;
                 lParam: LPARAM): LRESULT stdcall;
var
    hChildWnd: HWND;
    
    
    function SetCustomBtnLabel(BtnClass: Cardinal;
                               BtnCaption: String): Boolean;
    var
        BtnNewLabel: PWideChar;
    begin
    
        if BtnCaption <> '' then
        begin
            BtnNewLabel := PChar(BtnCaption);
            // la lnea anterior la uso cmo:
            // BtnNewLabel := PWideChar(BtnCaption);
            // en Delphi 2009. No coloco el condicional xq lo he olvidado
            // el nmero de versin interna XD
            
            Result := SetDlgItemText(hChildWnd, BtnClass, BtnNewLabel);
        end;
        
    end;

begin
  
    if (nCode = HCBT_ACTIVATE) then
    begin
    
        // cuando nCode es HCBT_ACTIVATE, indica que el dialogo est a
        // punto de ser activado. El dialogo es indicado en wParam.
        
        hChildWnd := HWND(wParam);
        
        with fCustomBtnsLabels do
        begin
            if (GetDlgItem(hChildWnd, IDABORT) <> 0) then
                SetCustomBtnLabel(IDABORT, ABORT);
                
            if (GetDlgItem(hChildWnd, IDCANCEL) <> 0) then
                SetCustomBtnLabel(IDCANCEL, CANCEL);
                
            if (GetDlgItem(hChildWnd, IDCONTINUE) <> 0) then
                SetCustomBtnLabel(IDCONTINUE, CONTINUE);

            if (GetDlgItem(hChildWnd, IDIGNORE) <> 0) then
                SetCustomBtnLabel(IDIGNORE, IGNORE);
                
            if (GetDlgItem(hChildWnd, IDNO) <> 0) then
                SetCustomBtnLabel(IDNO, NO);
                
            if (GetDlgItem(hChildWnd, IDOK) <> 0) then
                SetCustomBtnLabel(IDOK, OK);
                
            if (GetDlgItem(hChildWnd, IDRETRY) <> 0) then
                SetCustomBtnLabel(IDRETRY, RETRY);
   
            if (GetDlgItem(hChildWnd, IDTRYAGAIN) <> 0) then
                SetCustomBtnLabel(IDTRYAGAIN, TRYAGAIN);
                
            if (GetDlgItem(hChildWnd, IDYES) <> 0) then
                SetCustomBtnLabel(IDYES, YES);
        end;  

        // exit CBT hook
        UnhookWindowsHookEx(hhk);
    
    end
    else
       CallNextHookEx(hhk, nCode, wParam, lParam);
       
    Result := 0;
        
end; // CBTProc

function CBTMessagebox(hWnd: HWND;
                       lpText: PChar;
                       lpCaption: PChar;
                       uType: Cardinal;
                       CustomBtnCaptions: TCBTCustomBtnLabels): Cardinal;
begin

    hhk := SetWindowsHookEx(WH_CBT,
                            @CBTProc,
                            0,
                            GetCurrentThreadId());

    fCustomBtnsLabels := CustomBtnCaptions;  // <-- para uso en CBTProc function!

    Result := MessageBox(hwnd, lpText, lpCaption, uType);

end;

end.